@@ -9,7 +9,7 @@ import weakref
 
 from fsspec.spec import AbstractBufferedFile
 from fsspec.utils import infer_storage_options, tokenize
-from fsspec.asyn import AsyncFileSystem, sync, sync_wrapper
+from fsspec.asyn import AsyncFileSystem, sync, sync_wrapper, maybe_sync
 
 import aiobotocore
 import botocore
@@ -206,12 +206,23 @@ class S3FileSystem(AsyncFileSystem):
             try:
                 return await method(**additional_kwargs)
             except S3_RETRYABLE_ERRORS as e:
+                logger.debug("Retryable error: %s" % e)
                 err = e
                 await asyncio.sleep(min(1.7**i * 0.1, 15))
             except Exception as e:
+                logger.debug("Nonretryable error: %s" % e)
                 err = e
                 break
-        raise translate_boto_error(err)
+        if "'coroutine'" in str(err):
+            # aiobotocore internal error - fetch original botocore error
+            tb = err.__traceback__
+            while tb.tb_next:
+                tb = tb.tb_next
+            try:
+                await tb.tb_frame.f_locals['response']
+            except Exception as e:
+                err = e
+        raise translate_boto_error(err) from err
 
     call_s3 = sync_wrapper(_call_s3)
 
@@ -301,7 +312,7 @@ class S3FileSystem(AsyncFileSystem):
             config_kwargs["signature_version"] = UNSIGNED
         conf = AioConfig(**config_kwargs)
         if self.session is None:
-            self.session = aiobotocore.get_session(**self.kwargs)
+            self.session = aiobotocore.AioSession(**self.kwargs)
         s3creator = self.session.create_client('s3', config=conf, **init_kwargs, **client_kwargs)
         self._s3 = await s3creator.__aenter__()
         self._kwargs_helper = ParamKwargsHelper(self.s3)
@@ -393,17 +404,17 @@ class S3FileSystem(AsyncFileSystem):
                       s3_additional_kwargs=kw, cache_type=cache_type,
                       autocommit=autocommit, requester_pays=requester_pays)
 
-    async def _lsdir(self, path, refresh=False, max_items=None):
+    async def _lsdir(self, path, refresh=False, max_items=None, delimiter="/"):
         bucket, prefix, _ = self.split_path(path)
         prefix = prefix + '/' if prefix else ""
-        if path not in self.dircache or refresh:
+        if path not in self.dircache or refresh or delimiter is None:
             try:
                 logger.debug("Get directory listing page for %s" % path)
                 pag = self.s3.get_paginator('list_objects_v2')
                 config = {}
                 if max_items is not None:
                     config.update(MaxItems=max_items, PageSize=2 * max_items)
-                it = pag.paginate(Bucket=bucket, Prefix=prefix, Delimiter='/',
+                it = pag.paginate(Bucket=bucket, Prefix=prefix, Delimiter=delimiter,
                                   PaginationConfig=config, **self.req_kw)
                 files = []
                 dircache = []
@@ -424,15 +435,31 @@ class S3FileSystem(AsyncFileSystem):
             except ClientError as e:
                 raise translate_boto_error(e) from e
 
-            self.dircache[path] = files
+            if delimiter:
+                self.dircache[path] = files
             return files
         return self.dircache[path]
 
+    async def _find(self, path, maxdepth=None, withdirs=None, detail=False):
+        bucket, key, _ = self.split_path(path)
+        if not bucket:
+            raise ValueError("Cannot traverse all of S3")
+        out = await self._lsdir(path, delimiter="")
+        if not out and key:
+            try:
+                out = [await self._info(path)]
+            except FileNotFoundError:
+                out = []
+        if detail:
+            return {o['name']: o for o in out}
+        return [o['name'] for o in out]
+
+    find = sync_wrapper(_find)
+
     async def _mkdir(self, path, acl="", create_parents=True, **kwargs):
         path = self._strip_protocol(path).rstrip('/')
         bucket, key, _ = self.split_path(path)
-        if not key or create_parents:
-            # catch error if creating existing bucket?
+        if not key or (create_parents and not self.exists(bucket)):
             if acl and acl not in buck_acls:
                 raise ValueError('ACL not in %s', buck_acls)
             try:
@@ -457,7 +484,13 @@ class S3FileSystem(AsyncFileSystem):
     mkdir = sync_wrapper(_mkdir)
 
     def makedirs(self, path, exist_ok=False):
-        self.mkdir(path, create_parents=True)
+        try:
+            self.mkdir(path, create_parents=True)
+        except FileExistsError:
+            if exist_ok:
+                pass
+            else:
+                raise
 
     async def _rmdir(self, path):
         try:
@@ -518,19 +551,32 @@ class S3FileSystem(AsyncFileSystem):
 
     ls = sync_wrapper(_ls)
 
-    def exists(self, path):
+    async def _exists(self, path):
         if path in ['', '/']:
             # the root always exists, even if anon
             return True
         bucket, key, version_id = self.split_path(path)
         if key:
-            return super().exists(path)
-        else:
             try:
-                self.ls(path)
+                if self._ls_from_cache(path):
+                    return True
+            except FileNotFoundError:
+                return False
+            try:
+                await self._info(path, bucket, key, version_id=version_id)
                 return True
             except FileNotFoundError:
                 return False
+        elif self.dircache.get(bucket, False):
+            return True
+        else:
+            try:
+                await self.s3.list_objects_v2(MaxKeys=1, Bucket=bucket, **self.req_kw)
+                return True
+            except Exception:
+                return False
+
+    exists = sync_wrapper(_exists)
 
     def touch(self, path, truncate=True, data=None, **kwargs):
         """Create empty file or truncate"""
@@ -597,6 +643,9 @@ class S3FileSystem(AsyncFileSystem):
 
     async def _put_file(self, lpath, rpath, chunksize=50*2**20, **kwargs):
         bucket, key, _ = self.split_path(rpath)
+        if os.path.isdir(lpath) and key:
+            # don't make remote "directory"
+            return
         size = os.path.getsize(lpath)
         with open(lpath, 'rb') as f0:
             if size < 5 * 2**20:
@@ -654,7 +703,9 @@ class S3FileSystem(AsyncFileSystem):
                     break
                 f0.write(chunk)
 
-    async def _info(self, path, bucket, key, kwargs={}, version_id=None):
+    async def _info(self, path, bucket=None, key=None, kwargs={}, version_id=None):
+        if bucket is None:
+            bucket, key, version_id = self.split_path(path)
         try:
             out = await self._call_s3(self.s3.head_object, kwargs, Bucket=bucket,
                                       Key=key, **version_id_kw(version_id), **self.req_kw)
@@ -716,7 +767,7 @@ class S3FileSystem(AsyncFileSystem):
         should_fetch_from_s3 = (key and self._ls_from_cache(path) is None) or refresh
 
         if should_fetch_from_s3:
-            return sync(self.loop, self._info, path, bucket, key, kwargs, version_id)
+            return maybe_sync(self._info, self, path, bucket, key, kwargs, version_id)
         return super().info(path)
 
     def checksum(self, path, refresh=False):
@@ -1490,4 +1541,4 @@ def _fetch_range(fs, bucket, key, version_id, start, end,
                       Range='bytes=%i-%i' % (start, end - 1),
                       **version_id_kw(version_id),
                       **req_kw)
-    return sync(fs.loop, resp['Body'].read)
+    return maybe_sync(resp['Body'].read, fs)
