@@ -1563,7 +1563,7 @@ def test_modified(s3):
 
     # Test bucket
     with pytest.raises(IsADirectoryError):
-        modified = s3.modified(path=test_bucket_name)
+        s3.modified(path=test_bucket_name)
 
 
 @pytest.mark.skipif(sys.version_info < (3, 7), reason="no asyncio.run in py36")
@@ -1571,6 +1571,7 @@ def test_async_s3(s3):
     async def _():
         s3 = S3FileSystem(anon=False,
                           asynchronous=True,
+                          loop=asyncio.get_running_loop(),
                           client_kwargs={"region_name": "eu-central-1",
                                          "endpoint_url": endpoint_uri})
 
@@ -1581,8 +1582,22 @@ def test_async_s3(s3):
         with pytest.raises(RuntimeError):
             await s3._cat_file(fn)
 
-        await s3.connect()  # creates client
+        s3.connect()  # creates client
 
         assert await s3._cat_file(fn) == data
 
-    asyncio.run(_())
\ No newline at end of file
+        with s3.open(fn, 'rb') as f:
+            assert f.read() == data
+
+        await s3._s3.close()
+
+    asyncio.run(_())
+
+
+def test_via_fsspec(s3):
+    import fsspec
+    s3.mkdir('mine')
+    with fsspec.open('mine/oi', "wb") as f:
+        f.write(b"hello")
+    with fsspec.open("mine/oi", 'rb') as f:
+        assert f.read() == b"hello"
